-- customServicesCLI.lua
-- Samer Sayeed  
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 03dec13,nik Added CLI support for Service Group
-- 27dec10,rac Modified all functions to support present changes 
-- 26dec08,ss  written
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/fwView"

-- this routine reads in a current access control rule configuration into a 
-- configuration object. Or wii just initialize the defaults in case a new
-- rule is getting added.
function customServicesCfgInit(args)
    local rowId = args[1]
    if (rowId == nil) then
	    configRow = {}
        configRow["Services.TypeOfService"] = "0"
        configRow["Services.IsDefault"] = "0"
    else
        configRow= db.getRow("Services","_ROWID_",rowId)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1, {}
        end
    end
    return configRow["Services._ROWID_"], configRow
end

-- The routine saves the modified/ newly added access control rule
function customServicesCfgSave(configRow)
    local errorFlag = "OK"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "Services"

    if (configRow["Services._ROWID_"] == nil) then
	configRow["Services.OID"]=nil
	if(configRow["Services.Protocol"] == "1" or configRow["Services.Protocol"] == "58")then
	configRow["Services.DestinationPortEnd"]=nil
	end
	configRow["Services.IsDefault"]="0"
        errorFlag, statusCode = fwView.serviceConfig(configRow, "-1", "add")
    else
        local name = db.getAttribute("Services", "_ROWID_", configRow["Services._ROWID_"], "ServiceName")
        if (configRow["Services.ServiceName"] ~= name) then
            printCLIError("Service Name can't be edit")
            return false
        end
        errorFlag, statusCode = fwView.serviceConfig(configRow, configRow["Services._ROWID_"], "edit")
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
        statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    printCLIError(statusMessage)
    return errorFlag, statusMessage
end

-- The routine saves the modified/ newly added access control rule
function customServicesCfgDel(args)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "Services"
    configRow= db.getRow("Services","_ROWID_",args[1])
    if (configRow == nil) then
        printCLIError("Row doesnt exist")
        return errorFlag, statusMessage
    end
    local defServices = db.getAttribute("Services","_ROWID_",args[1],"IsDefault")
    if(defServices == "1") then
        printCLIError("Default Services cann't be deleted")
        return errorFlag, statusMessage
    end
    local rows = {}
    rows["rowid"] = configRow["Services._ROWID_"]
    errorFlag, statusCode = fwView.deleteServices(rows)
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    printCLIError(statusMessage)
    return errorFlag, statusMessage
end

function customServicesCfgInputVal(configRow)
    if(configRow["Services.IsDefault"] == "1") then
        printCLIError ("Invalid Row Id\n")
        return false
    end
    if (configRow["Services.ServiceName"] == "" or configRow["Services.ServiceName"] == nil) then
        printCLIError ("Enter unique service name\n")
        return false
    end
    if (configRow["Services.Protocol"] == nil) then
        printCLIError ("Enter valid protocol name\n")
        return false
    end
    if ((configRow["Services.Protocol"] == "1" or configRow["Services.Protocol"] == "58") and configRow["Services.DestinationPortStart"] == nil) then
        printCLIError ("Enter valid ICMP Port\n")
        return false
    elseif (configRow["Services.Protocol"] == "1" and configRow["Services.DestinationPortStart"] ~= nil) then
        if (tonumber(configRow["Services.DestinationPortStart"]) < 0 or tonumber(configRow["Services.DestinationPortStart"]) > 40) then
        printCLIError ("Enter valid ICMP Port between 0-40\n")
        return false
        end
    elseif (configRow["Services.Protocol"] == "58" and configRow["Services.DestinationPortStart"] ~= nil) then
        if (tonumber(configRow["Services.DestinationPortStart"]) < 1 or tonumber(configRow["Services.DestinationPortStart"]) > 255) then
        printCLIError ("Enter valid ICMP Port between 1-255\n")
        return false
        end
    end
    if (configRow["Services.Protocol"] == "6" or configRow["Services.Protocol"] == "17") then 
        if(configRow["Services.DestinationPortStart"] == "" or configRow["Services.DestinationPortStart"] == nil) then
            printCLIError ("Enter valid Start Port Value")
            return false
        end
        if(configRow["Services.DestinationPortEnd"] == "" or configRow["Services.DestinationPortEnd"] == nil) then
            printCLIError ("Enter valid End Port Value")
            return false
        end
        if (tonumber(configRow["Services.DestinationPortEnd"]) < tonumber(configRow["Services.DestinationPortStart"])) then
            printCLIError ("Invalid range of ports.End service port should be greater than start service port")
            return false
        end
    end
    return true
end

function customServicesCfgGet()
    local serviceTable = db.getTable ("Services")
    local resultTab = {} -- table to store DHCP leased clients list
    local serviceRow = nil
    local rowId = "0"

    printLabel ("List of Available Custom Services")    
    for col, val in pairs(serviceTable) do
        rowId = rowId+1
        local serviceRow = serviceTable[rowId]
	    if (rowId ~= nil) then
            if(serviceRow["Services.IsDefault"] == "0") then
	            resTab.insertField (resultTab, "ROW ID", serviceRow["Services._ROWID_"] or "")
        	    resTab.insertField (resultTab, "Name", serviceRow["Services.ServiceName"] or "")
                if(serviceRow["Services.Protocol"] == "1")then
                    resTab.insertField (resultTab, "Type", "ICMP")
                elseif(serviceRow["Services.Protocol"] == "6")then
                    resTab.insertField (resultTab, "Type", "TCP")
                elseif(serviceRow["Services.Protocol"] == "17")then
                    resTab.insertField (resultTab, "Type", "UDP")
                elseif(serviceRow["Services.Protocol"] == "58")then
                    resTab.insertField (resultTab, "Type", "ICMPv6")
                end -- if Type
                if(serviceRow["Services.Protocol"] == "1" or serviceRow["Services.Protocol"] == "58")then
                    resTab.insertField (resultTab, "ICMP Type / Port Range", serviceRow["Services.DestinationPortStart"] or "")
                else
                    local portRange = serviceRow["Services.DestinationPortStart"] .. "-" .. serviceRow["Services.DestinationPortEnd"]
                    resTab.insertField (resultTab, "ICMP Type / Port Range", portRange or "")
                end -- if Type
                --[[if(serviceRow["Services.TypeOfService"] == "0")then
                    resTab.insertField (resultTab, "QoS", "Normal")
                elseif(serviceRow["Services.TypeOfService"] == "2")then
                    resTab.insertField (resultTab, "QoS", "Minimise-Cost")
                elseif(serviceRow["Services.TypeOfService"] == "4")then
                    resTab.insertField (resultTab, "QoS", "Maximise Reliability")
                elseif(serviceRow["Services.TypeOfService"] == "8")then
                    resTab.insertField (resultTab, "QoS", "Maximise Throughput")
                elseif(serviceRow["Services.TypeOfService"] == "16")then
                    resTab.insertField (resultTab, "QoS", "Minimise Delay")
                end ]]-- if Type
            end -- if Default
        end -- if rowid nil
    end -- for 
    resTab.print (resultTab, 0)
end

function qosProfileCfgInit(args)
    local rowId = args[1]
    if (rowId == nil) then
	    configRow = {}
        configRow["qosProfile.Remark"] = "0"
        configRow["qosProfile.Priority"] = "0"
        configRow["qosProfile._ROWID_"] = "-1"
    else
        configRow = db.getRow("qosProfile","_ROWID_",rowId)
        if (configRow == nil) then
           printCLIError("Row does not exist")          
           return -1, {}
        end
    end
    return configRow["qosProfile._ROWID_"], configRow
end

function qosProfileCfgInputVal(configRow)
    if (configRow["qosProfile.ProfileName"] == nil or configRow["qosProfile.ProfileName"] == "") then
        printCLIError("Please enter Profile Name")
        return false
    end
    if (configRow["qosProfile.Remark"] == "1") then
        if (configRow["qosProfile.qosType"] == nil or configRow["qosProfile.qosType"] == "") then
            printCLIError("Invalid qos_type")
            return false
        end
        if (configRow["qosProfile.Value"] == nil or configRow["qosProfile.Value"] == "") then
            printCLIError("Invalid qos_value")
            return false
        end
        if ((configRow["qosProfile.qosType"] == "0") and (tonumber(configRow["qosProfile.Value"]) < 0 or
            tonumber(configRow["qosProfile.Value"]) > 7)) then
            printCLIError("Invalid qos_value. Please enter a value between 0 - 7")
            return false
        end
        if ((configRow["qosProfile.qosType"] == "1") and (tonumber(configRow["qosProfile.Value"]) < 1 or
            tonumber(configRow["qosProfile.Value"]) > 63)) then
            printCLIError("Invalid qos_value. Please enter a value between 1 - 63")
            return false
        end
    elseif (configRow["qosProfile.Remark"] == "0") then
        if (configRow["qosProfile.qosType"] ~= nil and configRow["qosProfile.qosType"] ~= "") then
            printCLIError("Cannot set qos_type when remark is disable")
            return false
        end
        if (configRow["qosProfile.Value"] ~= nil and configRow["qosProfile.Value"] ~= "") then
            printCLIError("Cannot set qos_value when remark is disable")
            return false
        end
    end
    return true
end

function qosProfileCfgSave(configRow)
    local errorFlag = "OK"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "qosProfile"

    if (configRow["qosProfile._ROWID_"] == "-1") then
        errorFlag, statusCode = fwView.qosProfileConfig(configRow, "-1", "add")
    else
        local profileName = db.getAttribute("qosProfile", "_ROWID_", configRow["qosProfile._ROWID_"], "ProfileName")
        if (configRow["qosProfile.ProfileName"] ~= profileName) then
            printCLIError("Profile Name can't be edited")
            return false
        end
        errorFlag, statusCode = fwView.qosProfileConfig(configRow, configRow["qosProfile._ROWID_"], "edit")
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    print(statusMessage)
    return errorFlag, statusMessage
end

function qosProfileCfgDel(args)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "qosProfile"
    
    configRow= db.getRow("qosProfile","_ROWID_",args[1])
    if (configRow == nil) then
		printCLIError("Row doesn't exist")
		return false
    end
    local rows = {}
    rows["rowid"] = configRow["qosProfile._ROWID_"]
    errorFlag, statusCode = fwView.deleteQosProfiles(rows)
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    print(statusMessage)
    return errorFlag, statusMessage
end

function qosProfileCfgGet()
    local qosTable = db.getTable ("qosProfile")
    local resultTab = {} 
    local qosRow = nil
    local rowId = "0"

    printLabel ("List of QoS Profiles")    
    for col, val in pairs(qosTable) do
        rowId = rowId+1
        local qosRow = qosTable[rowId]
	    if (rowId ~= nil) then
	        resTab.insertField (resultTab, "ROW ID", qosRow["qosProfile._ROWID_"] or "")
        	resTab.insertField (resultTab, "Profile Name", qosRow["qosProfile.ProfileName"] or "")
            if(qosRow["qosProfile.Remark"] == "0")then
                resTab.insertField (resultTab, "QoS Type", "No Marking")
                resTab.insertField (resultTab, "QoS Value", "0")
            elseif(qosRow["qosProfile.Remark"] == "1")then
                if(qosRow["qosProfile.qosType"] == "0") then
                    resTab.insertField (resultTab, "QoS Type", "IP-Precedence")
                elseif(qosRow["qosProfile.qosType"] == "1") then
                    resTab.insertField (resultTab, "QoS Type", "DSCP")
                end
                resTab.insertField (resultTab, "QoS Value", qosRow["qosProfile.Value"])
            end
            if(qosRow["qosProfile.Priority"] == "0")then
                resTab.insertField (resultTab, "Priority", "Default")
            elseif(qosRow["qosProfile.Priority"] == "1")then
                resTab.insertField (resultTab, "Priority", "High")
            elseif(qosRow["qosProfile.Priority"] == "2")then
                resTab.insertField (resultTab, "Priority", "Medium High")
            elseif(qosRow["qosProfile.Priority"] == "3")then
                resTab.insertField (resultTab, "Priority", "Medium")
            elseif(qosRow["qosProfile.Priority"] == "4")then
                resTab.insertField (resultTab, "Priority", "Low")
            end
        end
    end
    resTab.print (resultTab, 0)
end

function ipGroupCfgInit(args)
    if (args[1] == nil) then
        configRow = {}
        configRow["ipGroup._ROWID_"] = "-1"
    else
        configRow = db.getRow("ipGroup","_ROWID_",args[1])
        if (configRow == nil) then
           printCLIError("Row does not exist")          
           return -1, {}
        end
    end
    return configRow["ipGroup._ROWID_"], configRow
end

function ipGroupCfgSave(configRow)
    local errorFlag = "OK"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "ipGroup"

    if (configRow["ipGroup._ROWID_"] == "-1") then
        errorFlag, statusCode = fwView.ipGroupsConfig(configRow, "-1", "add")
    else
        errorFlag, statusCode = fwView.ipGroupsConfig(configRow, configRow["ipGroup._ROWID_"], "edit")
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    print(statusMessage)
    return errorFlag, statusMessage
end

function ipGroupCfgInputVal(configRow)
    if (configRow["ipGroup.ipGroupName"] == "" or configRow["ipGroup.ipGroupName"] == nil) then
        printCLIError("Invalid ip_group_name")
        return false
    end
    if (configRow["ipGroup.ipGroupType"] == "" or configRow["ipGroup.ipGroupType"] == nil) then
        printCLIError("Invalid ip_group_type")
        return false
    end
    return true
end

function ipGroupCfgDel(args)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "ipGroup"
    
    configRow= db.getRow("ipGroup","_ROWID_",args[1])
    if (configRow == nil) then
		printCLIError("Row doesn't exist")
		return false
    end
    local rows = {}
    rows["rowid"] = configRow["ipGroup._ROWID_"]
    errorFlag, statusCode = fwView.deleteIpGroups(rows)
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    print(statusMessage)
    return errorFlag, statusMessage
end

function ipGroupCfgGet()
    local ipGroupTable = db.getTable ("ipGroup")
    local resultTab = {} 
    local ipGroupRow = nil
    local rowId = "0"

    printLabel ("Custom IP Groups Table")    
    for col, val in pairs(ipGroupTable) do
        rowId = rowId+1
        local ipGroupRow = ipGroupTable[rowId]
	    if (rowId ~= nil) then
	        resTab.insertField (resultTab, "ROW ID", ipGroupRow["ipGroup._ROWID_"] or "")
        	resTab.insertField (resultTab, "IP Group Name", ipGroupRow["ipGroup.ipGroupName"] or "")
            if(ipGroupRow["ipGroup.ipGroupType"] == "0")then
                resTab.insertField (resultTab, "IP Group Type", "LAN Group")
            elseif(ipGroupRow["ipGroup.ipGroupType"] == "1")then
                resTab.insertField (resultTab, "IP Group Type", "WAN Group")
            end
        end
    end
    resTab.print (resultTab, 0)
end

function ipGroupIpCfgInit(args)
    if (db.existsRow("ipGroup","ipGroupName",args[1])) then
        configRow = {}
        configRow["ipGroupIp.ipGroup"] = args[1]
    else
        printCLIError("This IP Group does not exist")          
        return -1, {}
    end
    return -1, configRow
end

function ipGroupIpCfgSave(configRow)
    local errorFlag = "OK"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "ipGroupIp"

    if(configRow["ipGroupIp.ipAddress"] == "" or configRow["ipGroupIp.ipAddress"] == nil or configRow["ipGroupIp.ipAddress"] =="0.0.0.0") then
        printCLIError("Invalid ip_address")
        return false
    end

    errorFlag, statusCode = fwView.ipGroupIP_Add(configRow, "-1", "add")
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    print(statusMessage)
    return errorFlag, statusMessage
end

function ipGroupIpDel(args)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "ipGroupIp"
    
    configRow= db.getRow("ipGroupIp","_ROWID_",args[1])
    if (configRow == nil) then
		printCLIError("Row doesn't exist")
		return false
    end
    local rows = {}
    rows["rowid"] = configRow["ipGroupIp._ROWID_"]
    errorFlag, statusCode = fwView.deleteIpGroupIP(rows)
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    print(statusMessage)
    return errorFlag, statusMessage
end

function ipGroupIpCfgGet()
    local ipGroupIpTable = db.getTable ("ipGroupIp")
    local resultTab = {} 
    local ipGroupIpRow = nil
    local rowId = "0"

    printLabel ("List of IP Group's IP Table")    
    for col, val in pairs(ipGroupIpTable) do
        rowId = rowId+1
        local ipGroupIpRow = ipGroupIpTable[rowId]
	    if (rowId ~= nil) then
	        resTab.insertField (resultTab, "ROW ID", ipGroupIpRow["ipGroupIp._ROWID_"] or "")
        	resTab.insertField (resultTab, "IP Group", ipGroupIpRow["ipGroupIp.ipGroup"] or "")
        	resTab.insertField (resultTab, "IP Address", ipGroupIpRow["ipGroupIp.ipAddress"] or "")
        end
    end
    resTab.print (resultTab, 0)
end

function serviceGroupCfgInit(args)
    if (args[1] == nil) then
        configRow = {}
        configRow["serviceGroup._ROWID_"] = "-1"
    else
        configRow = db.getRow("serviceGroup","_ROWID_",args[1])
        if (configRow == nil) then
           printCLIError("Row does not exist")          
           return -1, {}
        end
    end
    return configRow["serviceGroup._ROWID_"], configRow
end

function serviceGroupCfgSave(configRow)
    local errorFlag = "OK"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "serviceGroup"

    if (configRow["serviceGroup._ROWID_"] == "-1") then
        errorFlag, statusCode = fwView.serviceGroupConfig(configRow, "-1", "add")
    else
        errorFlag, statusCode = fwView.serviceGroupConfig(configRow, configRow["serviceGroup._ROWID_"], "edit")
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    print(statusMessage)
    return errorFlag, statusMessage
end

function serviceGroupCfgInputVal(configRow)
    if (configRow["serviceGroup.serviceGroupName"] == "" or configRow["serviceGroup.serviceGroupName"] == nil) then
        printCLIError("Invalid service_group_name")
        return false
    end
    local serviceList = ''
    local serviceGroup = db.getRowsWhere("serviceGroupList","ServiceGroupName='"..configRow["serviceGroup.serviceGroupName"].."'")
    if(serviceGroup ~= nil) then
        local i = 0
        for k,v in pairs(serviceGroup) do
            i = i + 1
            local row = serviceGroup[i]
            if (serviceList == '') then
                serviceList = row["serviceGroupList.ServiceName"]
            else
                serviceList = serviceList..","..row["serviceGroupList.ServiceName"]
            end
        end
    end
    configRow["serviceGroup.serviceList"] = serviceList
    if (configRow["serviceGroup.ServiceNameDel"] ~= "" and configRow["serviceGroup.ServiceNameDel"] ~= nil) then
        local serviceListCur = {}
        serviceListCur = util.split (configRow["serviceGroup.serviceList"], ",")
        local serviceListNew = ''
        local j = 0
        for l,m in pairs(serviceListCur) do
            j = j + 1
            local serviceCur = serviceListCur[j]
            if (serviceCur ~= nil) then
                local servicesDel = {}
                servicesDel = util.split (configRow["serviceGroup.ServiceNameDel"], ",")
                local i = 0
                for k,v in pairs(servicesDel) do
                    i = i + 1
                    local serviceToDel = servicesDel[i]
                    if (serviceToDel ~= nil and serviceToDel ~= serviceCur) then
                        if (serviceListNew == '') then
                            serviceListNew = serviceCur
                        else
                            serviceListNew = serviceListNew..","..serviceCur
                        end
                    end
                end
            end
        end
        configRow["serviceGroup.serviceList"] = serviceListNew
    end
    if (configRow["serviceGroup.ServiceName"] ~= "" and configRow["serviceGroup.ServiceName"] ~= nil) then
        local serviceTable
        local match = 0
        if (configRow["serviceGroup._ROWID_"] == "-1") then
            serviceTable = db.getTable ("Services")
        else
            local where = "ServiceName NOT IN (select ServiceName from serviceGroupList where ServiceGroupName = '" .. configRow["serviceGroup.serviceGroupName"] .. "')"
            serviceTable =  db.getRowsWhere ("Services",where)
        end
        if (serviceTable ~= nil) then
            local services = {}
            services = util.split (configRow["serviceGroup.ServiceName"], ",")
            local i = 0
            for k,v in pairs(services) do
                i = i + 1
                local serviceName = services[i]
                if (serviceName ~= nil) then
                    for k, v in pairs (serviceTable) do
                        if (v["Services.Protocol"] == '6' or v["Services.Protocol"] == '17') then
                            if (serviceName == v["Services.ServiceName"]) then
                                match = 1
                                if (configRow["serviceGroup.serviceList"] ~= '') then
                                    configRow["serviceGroup.serviceList"] = configRow["serviceGroup.serviceList"]..","..serviceName
                                else
                                    configRow["serviceGroup.serviceList"] = serviceName
                                end
                            end
                        end
                    end
                end
                if (match == 0) then
                    printCLIError("Invalid Service Name: '"..serviceName.."',Please enter from following Service Names seperated by comma")
                    if (serviceTable ~= nil) then
                        for k, v in pairs (serviceTable) do
                            if (v["Services.Protocol"] == '6' or v["Services.Protocol"] == '17') then
                                print("'"..v["Services.ServiceName"].."'")
                            end
                        end
                    end
                    return false
                end
            end
        end
    end
    return true
end

function serviceGroupCfgDel(args)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "serviceGroup"
    
    configRow= db.getRow("serviceGroup","_ROWID_",args[1])
    if (configRow == nil) then
		printCLIError("Row doesn't exist")
		return false
    end
    local rows = {}
    rows["rowid"] = configRow["serviceGroup._ROWID_"]
    errorFlag, statusCode = fwView.deleteServiceGroups(rows)
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    print(statusMessage)
    return errorFlag, statusMessage
end

function serviceGroupCfgGet()
    local serviceGroupTable = db.getTable ("serviceGroup")
    local resultTab = {} 
    local serviceGroupRow = nil
    local rowId = "0"

    printLabel ("Custom Service Groups Table")    
    for col, val in pairs(serviceGroupTable) do
        rowId = rowId+1
        local serviceGroupRow = serviceGroupTable[rowId]
	    if (rowId ~= nil) then
	        resTab.insertField (resultTab, "ROW ID", serviceGroupRow["serviceGroup._ROWID_"] or "")
        	resTab.insertField (resultTab, "Service Group Name", serviceGroupRow["serviceGroup.serviceGroupName"] or "")
            local serviceList = ''
            local serviceGroup = db.getRowsWhere("serviceGroupList","ServiceGroupName='"..serviceGroupRow["serviceGroup.serviceGroupName"].."'")
            if(serviceGroup ~= nil) then
                local i = 0
                for k,v in pairs(serviceGroup) do
                    i = i + 1
                    local row = serviceGroup[i]
                    if (serviceList == '') then
                        serviceList = row["serviceGroupList.ServiceName"]
                    else
                        serviceList = serviceList..","..row["serviceGroupList.ServiceName"]
                    end
                end
            end
            resTab.insertField (resultTab, "List of Services", serviceList or "")
        end
    end
    resTab.print (resultTab, 0)
end
