--[[
#### Hussain Vali
#### TeamF1
#### www.TeamF1.com
#### Mar 28, 2008

#### File: dhcp.lua
#### Description: dhcp functions

#### Revisions:
01b,30sep13,ksr fixed SPR 39379
01a,19Jul08,shv added DHCP Relay Gateway configuration function.
]]--


--************* Requires *************
require "teamf1lualib/ifDev"

--************* Initial Code *************
--package dhcp
dhcp = {}
--************* Functions *************
-- dhcp config
function dhcp.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    if (operation == "add") then
        return nil
    elseif (operation == "edit") then
        valid = dhcp.updateConfig(inputTable, rowid)
    elseif (operation == "delete") then
        return nil
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DHCP_CONFIG_FAILED"
    end
end

-- Updating Configurations
function dhcp.updateConfig (inputTable, rowid)
    -- updating DHCP Configurations
    local inTable = {}
    inTable["dhcp.dhcpEnabled"] = inputTable["dhcp.dhcpEnabled"]
    if (inputTable["dhcp.dhcpEnabled"] ~= 0) then
        inTable["dhcp.domainName"] = inputTable["dhcp.domainName"]
        inTable["dhcp.startAddress"] = inputTable["dhcp.startAddress"]
        inTable["dhcp.endAddress"] = inputTable["dhcp.endAddress"]
        inTable["dhcp.primaryDnsServer"] = inputTable["dhcp.primaryDnsServer"]
        inTable["dhcp.secondaryDnsServer"] = inputTable["dhcp.secondaryDnsServer"]
        --inTable["dhcp.primaryTftpServer"] = inputTable["dhcp.primaryTftpServer"]
        --inTable["dhcp.secondaryTftpServer"] = inputTable["dhcp.secondaryTftpServer"]
        inTable["dhcp.winsServer"] = inputTable["dhcp.winsServer"]
        inTable["dhcp.leaseTime"] = inputTable["dhcp.leaseTime"]
        inTable["dhcp.enableDnsProxy"] = inputTable["dhcp.enableDnsProxy"]
    end

    -- validate
    if (dhcp.inputvalidate(inTable)) then
        return db.update("dhcp",inTable, rowid)
    end
end

-- dhcp inputvalidate
function dhcp.inputvalidate (inputTable)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

function dhcp.fixedIpConfig (inputTable, rowid, operation, DBTable)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local whereQry = ""

    if (operation ~= "delete") then
        whereQry = "MacAddr = '" ..inputTable["DhcpfixedIpAddress.MacAddr"].. "' AND GroupName = '" ..inputTable["DhcpfixedIpAddress.GroupName"].."'"
    end

    if(operation == "add") then
    	if(db.existsRow(DBTable,"IpAddr",inputTable["DhcpfixedIpAddress.IpAddr"])) then
    		return "ERROR", "DUPLICATE_IPADDR"
    	end
        --if (inputTable["DhcpfixedIpAddress.xmlId"] == nil) then
            maxId = db.columnMaxValueWhere("DhcpfixedIpAddress", "xmlId", "MacAddr IS NOT NULL")
            if(maxId == "" or maxId == nil) then
                maxId = -1
            end
            newid = tonumber(maxId) + 1
            if (db.existsRow ("AvailableLanHost","OID",tostring(newid))) then
                maxid = db.columnMaxValueWhere("AvailableLanHost", "OID", "OID IS NOT NULL")
                newid = tonumber(maxId) + 1
            end
            inputTable["DhcpfixedIpAddress.xmlId"] = tostring(newid)
        --end
        if(db.existsRowWhere (DBTable, whereQry)) then
        return "ERROR", "DUPLICATE_MACADDR"
        end
    elseif(operation == "edit") then
    	local dhcpFixedRowId = db.getAttribute("DhcpfixedIpAddress","IpAddr",inputTable["DhcpfixedIpAddress.IpAddr"],"_ROWID_")
    	if(dhcpFixedRowId ~= nil and dhcpFixedRowId ~= rowid) then
    		return "ERROR", "DUPLICATE_IPADDR"
    	end
        if (inputTable["DhcpfixedIpAddress.xmlId"] == nil) then
            maxId = db.columnMaxValueWhere("DhcpfixedIpAddress", "xmlId", "MacAddr IS NOT NULL")
            if(maxId == "" or maxId == nil) then
                maxId = -1
            end
            newid = tonumber(maxId) + 1
            inputTable["DhcpfixedIpAddress.xmlId"] = tostring(newid)
        end
        dhcpFixedRow = db.getRowWhere("DhcpfixedIpAddress",whereQry)
        if (dhcpFixedRow ~= nil and dhcpFixedRow ~= "")  then
            if(dhcpFixedRow["DhcpfixedIpAddress._ROWID_"] ~= nil and dhcpFixedRow["DhcpfixedIpAddress._ROWID_"] ~= rowid) then
                return "ERROR", "DUPLICATE_MACADDR"
            end
        end
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local lanHostTbl = {}
    lanHostTbl["LanHost.IPAddr"] = inputTable["DhcpfixedIpAddress.IpAddr"]
    lanHostTbl["LanHost.GroupName"] = inputTable["DhcpfixedIpAddress.GroupName"]
    if (operation == "add") then
        valid = db.insert("LanHost",lanHostTbl)
        if(valid) then
            valid = db.insert("DhcpfixedIpAddress", inputTable)
        end
    elseif (operation == "edit") then
	if(DBTable == "AvailableLanHost") then
	   if(db.existsRow("DhcpfixedIpAddress","IpAddr",inputTable["DhcpfixedIpAddress.IpAddr"])) then
    		return "ERROR", "DUPLICATE_IPADDR"
    	end
    	if(db.existsRow("DhcpfixedIpAddress","MacAddr",inputTable["DhcpfixedIpAddress.MacAddr"])) then
    		return "ERROR", "DUPLICATE_MACADDR"
    	end
	    valid = db.insert("LanHost",lanHostTbl)
            if(valid) then
                valid = db.insert("DhcpfixedIpAddress", inputTable)
            end
	else
            valid = db.update("LanHost", lanHostTbl, rowid)
            if(valid) then
                valid = db.update("DhcpfixedIpAddress", inputTable, rowid)
            end
        end
    elseif (operation == "delete") then
        local i = 0
        for k,v in pairs(inputTable) do
            i = i + 1
            local row = inputTable[i]
            valid = db.deleteRow("LanHost", "_ROWID_", v)
            if(valid) then
                db.deleteRow("DhcpfixedIpAddress", "_ROWID_", v)
            end
        end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DHCP_CONFIG_FAILED"
    end
end

function dhcp.dhcpConfigWrap (inputTable, logIfName)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

	if (inputTable["dhcp.dhcpEnabled"] == "1") then
		local maxDhcpIpRange = 16384 --2^14
		local dhcpRange = ifDev.ipAddrRange (inputTable["dhcp.startAddress"],inputTable["dhcp.endAddress"])

		if (dhcpRange ~= "-1") then
		if (tonumber(dhcpRange) > tonumber(maxDhcpIpRange)) then
        	return "ERROR", "DHCP_MAX_IP_RANGE_REACHED"
		end
		end
	end

    db.beginTransaction() --begin transaction
    local dhcpRelayUpdate = false
    local valid = false
    local where = "logicalIfName = '" .. logIfName .."'"
    local whr = "LogicalIfName = '" .. logIfName .."'"
    local dhcpsConfig = db.getRowWhere ("dhcp",where)
    local dhcpRelayConfig = db.getRowWhere ("DhcpRelay",whr)
    local dhcpLdapServerConfig = db.getRowWhere ("DhcpLdapServerInfo",whr)

    local dhcpRelayInTable = {}
    if (inputTable["dhcp.dhcpEnabled"] == "3") then
        inputTable["dhcp.dhcpEnabled"] = "0"
        dhcpRelayInTable["DhcpRelay.DhcpRelayStatus"] = "1"
        dhcpRelayInTable["DhcpRelay.RelayGateway"] = inputTable["DhcpRelay.RelayGateway"]
    elseif (inputTable["dhcp.dhcpEnabled"] == "0") then
        inputTable["dhcp.dhcpEnabled"] = "0"
        dhcpRelayInTable["DhcpRelay.DhcpRelayStatus"] = "0"
    else
        dhcpRelayInTable["DhcpRelay.DhcpRelayStatus"] = "0"
    end

    if (inputTable["ConfigPort.LogicalIfName"] ~= "DMZ") then
        inputTable["dhcp.dhcpEnabled"] = "0"
        dhcpRelayInTable["DhcpRelay.DhcpRelayStatus"] = "0"
    end

    local rowid = nil
    if (dhcpsConfig ~= nil) then
        rowid = dhcpsConfig["dhcp._ROWID_"]
    end
    if (rowid ~= nil) then
         valid = dhcp.updateConfig (inputTable, rowid)
    end
    
    if (valid and dhcpRelayConfig ~= nil) then
        rowid = dhcpRelayConfig["DhcpRelay._ROWID_"]
            if (valid) then
                valid = db.update("DhcpRelay", dhcpRelayInTable, rowid)
            end
    end
    local dhcpLdapServerInTable = {}
    dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapStatus"] = inputTable["DhcpLdapServerInfo.DhcpLdapStatus"]
    if(dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapStatus"] ~= "0") then
        dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapServerIP"] = inputTable["DhcpLdapServerInfo.DhcpLdapServerIP"]
        dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapSearchBase"] = inputTable["DhcpLdapServerInfo.DhcpLdapSearchBase"]
        dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapServerPort"] = inputTable["DhcpLdapServerInfo.DhcpLdapServerPort"]
    end
    if (valid and dhcpLdapServerConfig ~= nil) then
        rowid = dhcpLdapServerConfig["DhcpLdapServerInfo._ROWID_"]
            if (valid) then
                valid = db.update("DhcpLdapServerInfo", dhcpLdapServerInTable, rowid)
            end
        end    
    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DHCP_CONFIG_FAILED"
    end
end
