-- dmzCLI.lua
-- Ravindranath
-- TeamF1
-- www.TeamF1.com
-- Description
-- CLISH dmz LUA routines
require "teamf1lualib/lan"
require "teamf1lualib/dhcp"
require "teamf1lualib/ifDev"

--
--This routine initializes the DMZ configuration
function dmzCfgInit()
    local where = "LogicalIfName = 'DMZ' AND AddressFamily = 2"
	local rowid = db.existsRowWhere ("ifStatic", where)
    configRow = db.getRow("ifStatic", "_ROWID_", rowid)	
    local dhcpConfigRow = db.getRow ("dhcp", "logicalIfName", "DMZ")
    if (dhcpConfigRow ~= nil) then
	    configRow ["dhcp.dhcpEnabled"] = dhcpConfigRow["dhcp.dhcpEnabled"]
		if (configRow ["dhcp.dhcpEnabled"] ~= 0) then			
			configRow ["dhcp.startAddress"] = dhcpConfigRow ["dhcp.startAddress"]
			configRow ["dhcp.endAddress"] = dhcpConfigRow ["dhcp.endAddress"]
			configRow ["dhcp.primaryDnsServer"] = dhcpConfigRow ["dhcp.primaryDnsServer"]
			configRow ["dhcp.secondaryDnsServer"] = dhcpConfigRow ["dhcp.secondaryDnsServer"]			
			configRow ["dhcp.winsServer"] = dhcpConfigRow ["dhcp.winsServer"]
			configRow ["dhcp.leaseTime"] = dhcpConfigRow ["dhcp.leaseTime"]
			configRow ["dhcp.enableDnsProxy"] = dhcpConfigRow ["dhcp.enableDnsProxy"]
		end
	end
	
    local dhcpEnable = db.getRow("ConfigPort","_ROWID_","1")
    configRow ["ConfigPort.LogicalIfName"] = dhcpEnable ["ConfigPort.LogicalIfName"]
    local dhcpRelayConfig = db.getRow("DhcpRelay", "LogicalIfName", "DMZ")
    if (dhcpRelayConfig ~= nil) then
    	if (dhcpRelayConfig["DhcpRelay.DhcpRelayStatus"] == "1") then
    		configRow ["dhcp.dhcpEnabled"] = "3"
    	end
    	configRow ["DhcpRelay.RelayGateway"] = dhcpRelayConfig["DhcpRelay.RelayGateway"]
    end
    local dhcpLdapConfig = db.getRow("DhcpLdapServerInfo","LogicalIfName","DMZ")
    if (dhcpLdapConfig ~= nil) then
        if (configRow ["dhcp.dhcpEnabled"] ~= 0) then
            configRow ["DhcpLdapServerInfo.DhcpLdapStatus"] = dhcpLdapConfig ["DhcpLdapServerInfo.DhcpLdapStatus"]
            if (dhcpLdapConfig["DhcpLdapServerInfo.DhcpLdapStatus"] == "1") then
                configRow ["DhcpLdapServerInfo.DhcpLdapServerIP"] = dhcpLdapConfig ["DhcpLdapServerInfo.DhcpLdapServerIP"]
                configRow ["DhcpLdapServerInfo.DhcpLdapSearchBase"] = dhcpLdapConfig ["DhcpLdapServerInfo.DhcpLdapSearchBase"]
                configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] = dhcpLdapConfig ["DhcpLdapServerInfo.DhcpLdapServerPort"]
            end
        end
    end
	return -1,configRow

end

--
--This routine validates the DMZ configuration
function dmzCfgInputVal(configRow)
if (configRow ["ConfigPort.LogicalIfName"] == "" or
   configRow ["ConfigPort.LogicalIfName"] == nil )then
	printCLIError("Invalid DMZ status")
    return false
end
if(configRow ["ConfigPort.LogicalIfName"] == "DMZ") then
    if (configRow["dhcp.dhcpEnabled"] == "" or
        configRow["dhcp.dhcpEnabled"] == nil) then
        printCLIError("Invalid dhcp status")
        return false
    end
    if (configRow["ifStatic.StaticIp"] == "" or
        configRow["ifStatic.StaticIp"] == nil or
        configRow["ifStatic.StaticIp"] == "0.0.0.0") then
        printCLIError("Invalid DMZ IP address")
        return false
    end
    if (configRow["ifStatic.NetMask"] == "" or
        configRow["ifStatic.NetMask"] == nil or
        configRow["ifStatic.NetMask"] == "0.0.0.0") then
        printCLIError("Invalid DMZ subnet mask")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.startAddress"] == nil or 
         configRow["dhcp.startAddress"] == "" or 
         configRow["dhcp.startAddress"] == "0.0.0.0")) then
        printCLIError ("Invalid Start IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.endAddress"] == nil or 
         configRow["dhcp.endAddress"] == "" or 
         configRow["dhcp.endAddress"] == "0.0.0.0")) then
        printCLIError ("Invalid End IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1") then
        local status = ifDev.ipSubnet (configRow["ifStatic.StaticIp"],configRow["ifStatic.NetMask"],configRow["dhcp.startAddress"],configRow["dhcp.endAddress"])
        if (status ~= nil and status ~= "OK") then
            printCLIError (status)
            return false
        end
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.leaseTime"] == "" or
         configRow["dhcp.leaseTime"] == nil)) then
        printCLIError ("Invalid lease Time")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["DhcpLdapServerInfo.DhcpLdapStatus"] == "" or
         configRow["DhcpLdapServerInfo.DhcpLdapStatus"] == nil)) then
        printCLIError ("Invalid lease Time")
        return false
    end

    if (configRow["dhcp.dhcpEnabled"] == "3" and 
       (configRow["DhcpRelay.RelayGateway"] == nil or 
        configRow["DhcpRelay.RelayGateway"] == "" or 
        configRow["DhcpRelay.RelayGateway"] == "0.0.0.0")) then
        printCLIError ("Invalid relay gateway")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and
        configRow["DhcpLdapServerInfo.DhcpLdapStatus"] == "1" and
       (configRow["DhcpLdapServerInfo.DhcpLdapServerIP"] == nil or
        configRow["DhcpLdapServerInfo.DhcpLdapServerIP"] == "" or
        configRow["DhcpLdapServerInfo.DhcpLdapServerIP"] == "0.0.0.0")) then
        printCLIError ("Invalid Ldap server IP")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and
        configRow["DhcpLdapServerInfo.DhcpLdapStatus"] == "1" and
       (configRow ["DhcpLdapServerInfo.DhcpLdapSearchBase"] == nil or
        configRow ["DhcpLdapServerInfo.DhcpLdapSearchBase"] == "" )) then
        printCLIError ("Invalid Ldap search base")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and
        configRow["DhcpLdapServerInfo.DhcpLdapStatus"] == "1" and
       (configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] == nil or
        configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] > "65535" or
        configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] < "0" or
        configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] == "")) then
        printCLIError ("Invalid Ldap port")
        return false
    end
    if (configRow ["dhcp.enableDnsProxy"] == "" or
        configRow ["dhcp.enableDnsProxy"] == nil) then
        printCLIError ("Invalid DnsProxy")
        return false
    end
end
    local dhcpLdapRow = db.getRowWhere("DhcpLdapServerInfo","LogicalIfName='DMZ'") 
    if (configRow["dhcp.dhcpEnabled"] == "0" or configRow["ConfigPort.LogicalIfName"] ~= "DMZ") then
        local temp = ''
        if (configRow["ConfigPort.LogicalIfName"] ~= "DMZ") then
            temp = "DMZ"
        else
            temp = "Dhcp"
        end
        if (dhcpLdapRow["DhcpLdapServerInfo.DhcpLdapStatus"] ~= configRow["DhcpLdapServerInfo.DhcpLdapStatus"]) then
            printCLIError (temp.." is disable so can't enable/disable ldap")
            return false
        end
        if (configRow["DhcpLdapServerInfo.DhcpLdapServerIP"] ~= nil and
            (dhcpLdapRow["DhcpLdapServerInfo.DhcpLdapServerIP"] ~= configRow["DhcpLdapServerInfo.DhcpLdapServerIP"])) then
            printCLIError (temp.." is disable so can't edit ldap_serverip")
            return false
        end
        if (configRow["DhcpLdapServerInfo.DhcpLdapSearchBase"] ~= nil and
            (dhcpLdapRow["DhcpLdapServerInfo.DhcpLdapSearchBase"] ~= configRow["DhcpLdapServerInfo.DhcpLdapSearchBase"])) then
            printCLIError (temp.." is disable so can't edit ldap_search_base")
            return false
        end
        if (configRow["DhcpLdapServerInfo.DhcpLdapServerPort"] ~= nil and
            (dhcpLdapRow["DhcpLdapServerInfo.DhcpLdapServerPort"] ~= configRow["DhcpLdapServerInfo.DhcpLdapServerPort"])) then
            printCLIError (temp.." is disable so can't edit ldap_port")
            return false
        end
    end
    return true
end

--
--This routine saves the DMZ configuration
function dmzCfgSave(configRow)
	local errorFlag="OK"
	local statusCode=""
	local statusMessage=""
	if (configRow["dhcp.primaryDnsServer"] == "0.0.0.0") then configRow["dhcp.primaryDnsServer"] = '' end
	if (configRow["dhcp.secondaryDnsServer"] == "0.0.0.0") then configRow["dhcp.secondaryDnsServer"] = '' end
	if (configRow["dhcp.winsServer"] == "0.0.0.0") then configRow["dhcp.winsServer"] = '' end
    local ifstaticRows = db.getRowsWhere("ifStatic","AddressFamily = 2 and LogicalIfName!='DMZ'")
    local j = 0
    if (#ifstaticRows ~= 0) then
        for k,v in pairs(ifstaticRows) do
            j = j + 1
            local ifstatic = ifstaticRows[j]
            if (ifstatic["ifStatic.LogicalIfName"] == "WAN1") then
                local ipAddrRow = db.getRowWhere("ipAddressTable","addressFamily = 2 and LogicalIfName='WAN1'")
                if (ipAddrRow ~= nil) then
                    ifstatic["ifStatic.StaticIp"] = ipAddrRow["ipAddressTable.ipAddress"] or ""
                    ifstatic["ifStatic.NetMask"] = ipAddrRow["ipAddressTable.subnetMask"] or ""
                end
            end
            if (ifstatic ~= nil and configRow ["ConfigPort.LogicalIfName"] == "DMZ" and ifstatic["ifStatic.StaticIp"] ~= nil and
                ifstatic["ifStatic.StaticIp"] ~= "0.0.0.0" and ifstatic["ifStatic.StaticIp"] ~= "") then
			    errorFlag = ifDevLib.ipSubnet(ifstatic["ifStatic.StaticIp"],ifstatic["ifStatic.NetMask"],configRow["ifStatic.StaticIp"])
			    local var ="Error: Invalid Start IP Address. " .. configRow["ifStatic.StaticIp"] .." is out of subnet range"
		        if(errorFlag == var) then
                    errorFlag = ifDevLib.ipSubnet(configRow["ifStatic.StaticIp"],configRow["ifStatic.NetMask"],ifstatic["ifStatic.StaticIp"])
                    local err ="Error: Invalid Start IP Address. " .. ifstatic["ifStatic.StaticIp"] .." is out of subnet range"
                    if(errorFlag ~= err) then
                        printCLIError ("IP address conflicting with "..ifstatic["ifStatic.LogicalIfName"])
                        return false
                    end
                elseif(errorFlag ~= var) then
                    printCLIError ("IP address conflicting with "..ifstatic["ifStatic.LogicalIfName"])
                    return false
                end
	        end
        end
    end
    if (configRow ["ConfigPort.LogicalIfName"] == "DMZ") then
        --DMZ is Enabled.
        db.setAttribute("dmzLED","_ROWID_","1","ledStatus","1")
--        os.execute("/pfrm2.0/bin/wgpio 36 0")
    else
        --DMZ is Disabled.
        db.setAttribute("dmzLED","_ROWID_","1","ledStatus","0")
--        os.execute("/pfrm2.0/bin/wgpio 36 1")
    end

		local where = "LogicalIfName = 'DMZ' AND AddressFamily = 2"
		local rowid = db.existsRowWhere ("ifStatic", where)
		errorFlag, statusCode = lan.ipv4LanConfig(configRow, rowid, "edit")
		if (errorFlag == "OK") then
			db.save()
			errorFlag, statusCode = dhcp.dhcpConfigWrap (configRow, "DMZ")
		end
	--end

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
	return errorFlag,statusMessage

end

--
--This routine gets the DMZ configuration
function dmzSetupGet()
    local resultTab = {} -- table to store DMZ setup information
    local ifdevTable = nil
    local where = "LogicalIfName = 'DMZ' AND AddressFamily = 2"
    local rowid = db.existsRowWhere ("ifStatic", where)
    local ipv4LanRow = db.getRow("ifStatic", "_ROWID_", rowid )
    local LdapRow = db.getRow("DhcpLdapServerInfo","LogicalIfName","DMZ")
    printLabel ("DMZ Setup")
    local DMZRow = db.getRow("ConfigPort","_ROWID_","1")
  if ( DMZRow["ConfigPort.LogicalIfName"] == "DMZ") then
    if(ipv4LanRow ~= nil) then
        resTab.insertField (resultTab, "IPv4 Address", ipv4LanRow["ifStatic.StaticIp"] or "")
        resTab.insertField (resultTab, "Subnet Mask", ipv4LanRow["ifStatic.NetMask"]or "")   
    end

    resTab.insertField (resultTab, "DHCP Setup Configuration", "")    
    local whereDhcp = "logicalIfName = 'DMZ'"
    local rowidDhcp = db.existsRowWhere ("dhcp", whereDhcp)
    dhcpTable = db.getRow ("dhcp","_ROWID_", rowidDhcp)

    local whereDhcpRelay = "LogicalIfName = 'DMZ'"
    local rowidDhcpRelay = db.existsRowWhere ("DhcpRelay", whereDhcpRelay)
    dhcpRelayTable = db.getRow ("DhcpRelay","_ROWID_", rowidDhcpRelay)
    
    if (dhcpTable ~= nil and dhcpRelayTable ~= nil) then 
        if (dhcpTable ["dhcp.dhcpEnabled"] == "0" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "1") then
               resTab.insertField (resultTab, "DHCP Mode", "DHCP Relay")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "1" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "0") then
               resTab.insertField (resultTab, "DHCP Mode", "DHCP Server")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "0" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "0") then
               resTab.insertField (resultTab, "DHCP Mode", "None")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "1") then 
           resTab.insertField (resultTab, "Domain Name", 
	                      dhcpTable["dhcp.domainName"] or "")
           resTab.insertField (resultTab, "Starting IP Address", 
	                      dhcpTable["dhcp.startAddress"] or "")
           resTab.insertField (resultTab, "Ending IP Address", 
			      dhcpTable["dhcp.endAddress"] or "")
           resTab.insertField (resultTab, "Primary DNS Server", 
			      dhcpTable["dhcp.primaryDnsServer"] or "")
           resTab.insertField (resultTab, "Secondary DNS Server", 
			      dhcpTable["dhcp.secondaryDnsServer"] or "")
           resTab.insertField (resultTab, "WINS Server", 
			      dhcpTable["dhcp.winsServer"] or "")
           resTab.insertField (resultTab, "Lease Time in hrs ", 
			      dhcpTable["dhcp.leaseTime"] or "")
        elseif (dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "1") then
            resTab.insertField (resultTab, "Relay Gateway", 
			      dhcpRelayTable["DhcpRelay.RelayGateway"] or "")
        end
	if (dhcpTable ["dhcp.dhcpEnabled"] == "1" and LdapRow["DhcpLdapServerInfo.DhcpLdapStatus"] == "1") then
		resTab.insertField (resultTab,"LDAP Status", "Enabled")
		resTab.insertField (resultTab,"DhcpLdapServerIP",LdapRow["DhcpLdapServerInfo.DhcpLdapServerIP"])
		resTab.insertField (resultTab,"DhcpLdapSearchBase",LdapRow["DhcpLdapServerInfo.DhcpLdapSearchBase"])
		resTab.insertField (resultTab,"DhcpLdapServerPort",LdapRow["DhcpLdapServerInfo.DhcpLdapServerPort"])
	else
		resTab.insertField (resultTab,"LDAP Status", "Disabled")
	end


    end               
    IgmpTable = db.getRow ("Igmp","_ROWID_", 1)
--    resTab.insertField (resultTab, "DMZ Proxies", "")
    if (dhcpTable["dhcp.enableDnsProxy"] == "1") then 
        resTab.insertField (resultTab, "DNS Proxy", " Enabled")
    else 
        resTab.insertField (resultTab, "DNS Proxy", " Disabled")
    end
    resTab.print (resultTab, 0)
  else
	  printCLIError("DMZ Disabled.")
	  return false
  end
  end
