-- idsCLI.lua
-- Samer Sayeed
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 31jan09,ss  fixed cli related changes for spr 10442
-- 31jan09,ss  modified idsCfgSave routine to reboot device only when ids is enabled.
-- 26dec08,ss  written
--
-- Description
-- CLI interface set and get routines of ids and session settings

--require "teamf1lualib/ids"
require "teamf1lualib/fwView"

function idsCfgInit(args)
    local configRow = {}
    configRow = db.getRow("IdsConfiguration","_ROWID_", "1")

    return 1, configRow
end

function idsCfgSave(configRow)
    errorFlag = "ERROR"
    statusCode = ""
    statusMessage = ""
    local triesMax = 5 -- hard-coded
    local triesCur = 0
    local confirm = ""
    local reboot = "0"
    local message = "Enabling IDS reboots the device.\n"
    DBTable = "IdsConfiguration"
   
    local idsStatus = db.getAttribute ("IdsConfiguration", "IdsConfiguration._ROWID_", "1", "IdsStatus")
    if(configRow["IdsConfiguration.IdsStatus"] == "1" and idsStatus == "0") then
        printCLIError (message)
        while (confirm ~= "Y" and confirm ~= "N" and triesCur < triesMax) do
	    printCLIError ("Are you sure you want to proceed [Y/N]?")
	    confirm = io.stdin:read'*l'
	    triesCur = triesCur + 1
        end
        if (confirm == "N" or (triesCur == triesMax)) then
            errorFlag = "ERROR"
            statusMessage = "Action Cancelled"
            return errorFlag, statusMessage
        elseif(confirm == "Y") then reboot = "1" end
    end
    
    errorFlag, statusCode = ids.config(configRow, "1", "edit")
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    if (reboot == "1") then utilCLIReboot() end
    return errorFlag, statusMessage
end

function idsCfgVal(configRow)
    return true
end

function idsInfoGet()
    printLabel ("IDS")
    row = db.getRow("IdsConfiguration", "IdsConfiguration._ROWID_",1)
--  The IdsConfiguration table should have only one entry
    if(row == nil) then return
    end
    print ("IDS Settings")
    if (row["IdsConfiguration.IdsStatus"] == "0") then
       print( " Intrusion Detection  : Disabled")
    elseif(row["IdsConfiguration.IdsStatus"] == "1") then 
       print( " Intrusion Detection  : Enabled")
       --if (row["IdsConfiguration.BlackListingStatus"] == "0") then
       --    print( " Black Listing        : Disabled")
       --elseif (row["IdsConfiguration.BlackListingStatus"] == "1") then 
       --    print( " Black Listing        : Enabled")    
       --end
       --if (row["IdsConfiguration.BlockLogStatus"] == "0") then
       --    print( " Blocking Log         : Disabled")      
       --elseif (row["IdsConfiguration.BlockLogStatus"] == "1") then 
       --    print( " Blocking Log         : Enabled")    
       --end
       if (row["IdsConfiguration.IntrusionLogStatus"] == "0") then
           print( " Intrusion Log        : Disabled")      
       elseif(row["IdsConfiguration.IntrusionLogStatus"] == "1") then 
           print( " Intrusion Log        : Enabled")    
       end
    end
end

function sessionSettingsCfgSave(configRow)
    errorFlag = "ERROR"
    statusCode = ""
    statusMessage = "" 
    DBTable = "GlobalSettings"
    
    if (db.existsRow("GlobalSettings","_ROWID_","1")) then
        errorFlag, statusCode = fwView.fwSessionconfig (configRow, "1", "edit")
    else
	errorFlag, statusCode = fwView.fwSessionconfig (configRow, "-1", "add")	
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode

    return errorFlag, statusMessage
end

function sessionSettingsCfgInit(args)
    local configRow = {}
    configRow = db.getRow("GlobalSettings","_ROWID_", "1")
    if (configRow == nil) then
    	configRow = {}
        configRow["GlobalSettings._ROWID_"] = "-1"
    end 
    return configRow["GlobalSettings._ROWID_"], configRow
end

function sessionSettingsInfoGet()
    printLabel ("Session Settings")
    row = db.getRow("GlobalSettings", "GlobalSettings._ROWID_",1)

    --  The GlobalSettings table should have only one entry
     
--    print ("Maximum Unidentified Sessions:" .. row["GlobalSettings.MaxUnidentifiedSessions"])
--    print ("Maximum Half Open Sessions:" .. row["GlobalSettings.MaxHalfOpenSessions"])
    print ("TCP Session Timeout Duration:" .. row["GlobalSettings.TcpSessionTimoutDuration"] .. "(Secs)")
    print ("UDP Session Timeout Duration:" .. row["GlobalSettings.UdpSessionTimoutDuration"] .. "(Secs)")
    print ("ICMP Session Timeout Duration:" .. row["GlobalSettings.OtherSessionTimeoutDuration"] .. "(Secs)")
--    print ("TCP Session Cleanup Latency:" .. row["GlobalSettings.TcpSessionCleanupLatency"] .. "(Secs)")
end

function sessionLimitingCfgSave(configRow)
    errorFlag = "ERROR"
    statusCode = ""
    statusMessage = "" 
    DBTable = "SessionLimit"
    if (db.existsRow("SessionLimit","_ROWID_","1")) then
        errorFlag, statusCode = fwView.sesLimitConfig (configRow, "1", "edit")
    else
	errorFlag, statusCode = fwView.sesLimitConfig (configRow, "-1", "add")	
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    if (errorFlag == "OK") then 
        if (configRow["SessionLimit.SessLimitEnable"]=="0") then
        print("Session Limit feature disabled. Other configuration changes are not effective in disabled state.")
    end
end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode

    return errorFlag, statusMessage
end

function sessionLimitingCfgInit(args)
    local configRow = {}
    configRow = db.getRow("SessionLimit","_ROWID_", "1")
    if (configRow == nil) then
    	configRow = {}
        configRow["SessionLimit._ROWID_"] = "-1"
    else
        local connectionTimeType = configRow["SessionLimit.ConnectionTimeType"]
        if(connectionTimeType ~= nil and connectionTimeType == "0") then
            local blockSessionType = configRow["SessionLimit.BlocKNewSession"]
            if(blockSessionType == "1") then
                configRow["SessionLimit.TimePeriod1"] = configRow["SessionLimit.TimePeriod"]
            else
                configRow["SessionLimit.TimePeriod2"] = configRow["SessionLimit.TimePeriod"]
            end
        end
    end 
    return configRow["SessionLimit._ROWID_"], configRow
end

function sessionLimitingCfgInputVal(configRow)
    if (configRow["SessionLimit.SessLimitEnable"] == "1" and configRow["SessionLimit.ConnectionTimeType"] == "0") then
        if (configRow["SessionLimit.BlocKNewSession"] == nil or configRow["SessionLimit.BlocKNewSession"] == "") then
            printCLIError ("Invalid block_new_session type")
            return false
        end
        if ((configRow["SessionLimit.BlocKNewSession"] == "1") and (configRow["SessionLimit.TimePeriod1"] == nil or configRow["SessionLimit.TimePeriod1"] == "")) then
            printCLIError ("Invalid block_IP_to_add_new_session_for_time.")
            return false
        end
        if ((configRow["SessionLimit.BlocKNewSession"] == "2") and (configRow["SessionLimit.TimePeriod2"] == nil or configRow["SessionLimit.TimePeriod2"] == "")) then
            printCLIError ("Invalid block_IPs_all_connections_for_time.")
            return false
        end
    end
    if(configRow["SessionLimit.SessLimitEnable"]=="1") then
        if(configRow["SessionLimit.ConnLimitType"]=="0" and tonumber(configRow["SessionLimit.UserConnLimit"]) > 100) then
            printCLIError("Invalid user_limit range. For percentage of Max Sessions case, user_limit must be below 100")
            return false
        end
        if(configRow["SessionLimit.ConnLimitType"]=="1" and tonumber(configRow["SessionLimit.UserConnLimit"]) > 10000) then
            printCLIError("Invalid user_limit range. user_limit in 'Number Of Sessions' case of session limit must not exceed 10000")
            return false
        end
    end

end

function sessionLimitingInfoGet()
    printLabel ("Session Settings")
    row = db.getRow("SessionLimit", "SessionLimit._ROWID_",1)
    local SessionTimeoutRow = db.getRow("GlobalSettings","_ROWID_",1)
    --  The GlobalSettings table should have only one entry
    if ( row["SessionLimit.SessLimitEnable"] ~= nil ) then
	    if (row["SessionLimit.SessLimitEnable"] == "0") then  
    		print ("Session Limit Enable: ", "Disabled")
	    else
    		print ("Session Limit Enable: ", "Enabled")
		 if (row["SessionLimit.ConnLimitType"] ~= nil ) then 
             if(row["SessionLimit.ConnLimitType"]== "0") then print ("Connection Limit Type: ", "Percentage of Maximum Sessions") end
             if(row["SessionLimit.ConnLimitType"]== "1") then print ("Connection Limit Type: ", "Number Of Sessions") end
         end
    		if ( row["SessionLimit.UserConnLimit"] ~= nil ) then print ("User Connection Limit: ", row["SessionLimit.UserConnLimit"]) end
            if (row["SessionLimit.ConnectionTimeType"] ~= nil) then
                if(row["SessionLimit.ConnectionTimeType"] == "0") then print("Session Limit Control:", "When Single IP Exceeds") end
                if(row["SessionLimit.ConnectionTimeType"] == "1") then print("Session Limit Control:", "When Single IP Cannot Exceeds") end
            end
            --the conditions on time period is taken care in validation function
            if(row["SessionLimit.BlocKNewSession"] ~= nil) then 
                if(row["SessionLimit.BlocKNewSession"] == "1") then print("Block IP to Add New Session for time:", row["SessionLimit.TimePeriod"]) end
                if(row["SessionLimit.BlocKNewSession"] == "2") then print("Block IPs All Connections for time:", row["SessionLimit.TimePeriod"]) end
            end


	   end
    end
    if ( SessionTimeoutRow ) then
	    if ( SessionTimeoutRow["GlobalSettings.TcpSessionTimoutDuration"] ~= nil ) then  print ("TCP Session Timeout Duration: ", SessionTimeoutRow["GlobalSettings.TcpSessionTimoutDuration"] .. "(Secs)") end
	    if ( SessionTimeoutRow["GlobalSettings.UdpSessionTimoutDuration"] ~= nil ) then print ("UDP Session Timeout Duration: ", SessionTimeoutRow["GlobalSettings.UdpSessionTimoutDuration"] .. "(Secs)") end
	    if ( SessionTimeoutRow["GlobalSettings.OtherSessionTimeoutDuration"] ~= nil ) then print ("ICMP Session Timeout Duration: ", SessionTimeoutRow["GlobalSettings.OtherSessionTimeoutDuration"] .. "(Secs)") end
    end
end


