--[[
#### Hussain Vali
#### TeamF1
#### www.TeamF1.com
#### July 02, 2007

#### File: login.lua
#### Description: Login functions.

#### Revisions:
01f,07jan13 ksr, changes for spr 20681
01e,23jan12 su,  changes to get userAgent from HTTP_USER_AGENT.
01d,03jan12 su,  changes for spr 28894
01c,29mar11 su,  changes for spr 22465.
01b,18mar11 su , added changes to allow extern user not present in localdb and                   changes for 2Factor authentication.
01a 28sep10 bhj, added fix for 17262
]]--


--************* Requires *************

require "teamf1lualib/db"
require "sslvpnLuaLib"
require "timeLib"

--************* Initial Code *************

--package login
login = {}
--************* Functions *************
--Authenticate with given account data and account table.
--Returns false if account with matching info not exists.
--if account with matching info exists its call login function.
function login.login (tableInput)
    local username = tableInput["USERDBUsers.UserName"]
    local password = tableInput["USERDBUsers.Password"]
--  local userAgent = tableInput["Login.userAgent"]
	local userAgent = os.getenv("HTTP_USER_AGENT")

    local domainName = tableInput["USERDBDomains.Domainname"]
    local lockoutEnabled = db.getAttribute("USERDBUsers", "UserName", username, "LockoutEnable")     
    local maxLockoutAttempts = db.getAttribute("USERDBUsers", "UserName", username, "MaxLockoutAttempts")
    local userConfig = db.getRow ("USERDBUsers","UserName",username)
    local status = false
          
    if(domainName == nil) then
        return -1,"No Domains Have Been Configured For This Portal."
    end
    if (userConfig ~= nil) then
        local userGroupName = db.getAttribute("USERDBUsers","UserName", username, "GroupName")
        local userDomain = db.getAttribute("USERDBGroups","GroupName",userGroupName,"DomainName")
        local authenType = db.getAttribute("USERDBDomains","DomainName",domainName,"AuthenticationType")
        if(authenType == "local" and domainName ~= userDomain) then
               return -1, "User Login Failed"
        end
	local userType = db.getAttribute ("USERDBUsers", "UserName", username, "UserType") 
        local authType = db.getAttribute ("USERDBDomains","DomainName",domainName,"AuthenticationType")
        local localDbDisable = db.getRow ("SSLVPNUserLoginLocalDbStatus", "_ROWID_", "1") 
        if (localDbDisable ~= nil) then
        if (localDbDisable["SSLVPNUserLoginLocalDbStatus.LocalDbDisable"] == "1") then
            if (authType == "local") then
			    return 3, "Local Authentication Disabled By Admin."
            end
        end
        end
        
        local portalName = db.getAttribute ("USERDBDomains","DomainName",domainName,"PortalLayoutName")
        if (Page == SSL_PORTAL_LOGIN_PAGE) then
		local selectedPortalName = tableInput["Login.PortalName"]
            if (selectedPortalName == nil) then
                return 3, "Failed to find Portal Information"
            elseif (selectedPortalName ~= portalName) then
		if(userType == "3") then 
			return 3, "Admin User authentication Failed."
		elseif(userType == "4") then 
			return 3,"Guest User authentication Failed."
		elseif(userType == "0") then
                 	return 3, "SSLVPN User authentication Failed. Use the correct SSL portal URL to login."
		else
                 	return 3, "User authentication Failed. Use the correct SSL portal URL to login."
		end
            end
        else
            local defaultPortalName = db.getAttribute ("SSLVPNPortalLayout","DefaultPortal","2","PortalLayoutName")
            if (defaultPortalName == nil) then
                defaultPortalName = db.getAttribute ("SSLVPNPortalLayout","DefaultPortal","1","PortalLayoutName")
            end
            if (defaultPortalName ~= portalName) then
                return 3, "User authentication Failed. Only default portal's users should be allowed to login."
            end
        end
         if(userConfig ~= nil and domainName == userDomain) then
             local status, errMsg = sslvpnLuaLib.sslLoginPolicyVerify (username, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent)
             if (status == -1) then
                 return 3, errMsg
             end
         end

		if (lockoutEnabled == "1") then
			-- If the user is locked, we should display LOCKED message and block the login
			local filep = io.open("/var/log/" .. username .. ".lockTime", "r")
   			local status,upTime = timeLib.uptime()
   			if (filep ~= nil) then
	   			local lockTime = filep:read("*line")
       			if (lockTime ~= nil) then
            		local lockoutTime = (upTime) - (lockTime)
                	local lockPeriod = db.getAttribute ("USERDBUsers", "UserName", username, "LockPeriod")
                	if (lockoutTime < tonumber(lockPeriod)) then
                		result = false
                	    statusCode = "USER_LOCKED"
						return -1, statusCode
                	else 
                		os.execute("rm -rf /var/log/" .. username .. ".lockTime")
                	end
				end
			end
		end

	local httpReferer = login.parseReferer (os.getenv("HTTP_REFERER") or '')
        local loginState, uniqueVal, errorMsg, prev = sslvpnLuaLib.sslLogin (username, password, domainName, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent, SAPI.Request.servervariable("URL"), httpReferer)
        if (loginState == -1) then

		if (lockoutEnabled == "1") then
			-- If user fails to authenticate and failure count has reached max 
        	-- attempts allowed, user will be blocked for time period configured 
        	-- which will be calculated from last failed login attempt.
			local wherePart = "UserName = '" .. username .. "'"
       		local oldRow = db.existsRowWhere("UserLoginFailure", wherePart)
       		local fail_cnt = db.getAttribute("UserLoginFailure", "UserName", username, "FailureCount")
       		local tempTable = {}
       		tempTable["UserLoginFailure.UserName"] = username
       		if(fail_cnt ~= nil) then
	       		fail_cnt = fail_cnt + 1
                if (maxLockoutAttempts == tostring(fail_cnt)) then
                	-- update the lock time file and reset login failure count, 
                    -- everytime user reaches max login attempts allowed
                    os.execute("/pfrm2.0/bin/timeShow > /var/log/" .. username .. ".lockTime")
                    fail_cnt = 0 
                end
                tempTable["UserLoginFailure.FailureCount"] = fail_cnt
                if (oldRow) then
                	db.update("UserLoginFailure", tempTable, oldRow)
                end
	        elseif(fail_cnt == nil) then
            	tempTable["UserLoginFailure.FailureCount"] = 1
		        db.insert("UserLoginFailure", tempTable)
            end
		end

		if(userType == "3") then
            		return -1, "Invalid Password for Admin User."
		elseif(userType == "4") then
            		return -1, "Invalid Password for Guest User."
		elseif(userType == "0") then
			return -1, "Invalid Password for SSLVPN User."
		else
			return -1, "Invalid Password."
		end
        elseif (loginState == 0) then
            cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
			db.setAttribute("UserLoginFailure", "UserName", username, "FailureCount", 0)
            return 0,"OK",nil,uniqueVal
        elseif (loginState == -2) then
            return 1,errorMsg,tableInput
        elseif (loginState == -4) then
            cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
            return 2,errorMsg
        else
            return 3,errorMsg
        end
    else                
        local userType = "0" --user not present in local db	    
        local portalName = db.getAttribute ("USERDBDomains","DomainName",domainName,"PortalLayoutName")
        if (Page == SSL_PORTAL_LOGIN_PAGE) then
		local selectedPortalName = tableInput["Login.PortalName"]
            if (selectedPortalName == nil) then
                return 3, "Failed to find Portal Information"
            elseif (selectedPortalName ~= portalName) then
		if(userType == "3") then 
			return 3, "Admin User authentication Failed."
		elseif(userType == "4") then 
			return 3,"Guest User authentication Failed."
		elseif(userType == "0") then
                 	return 3, "SSLVPN User authentication Failed. Use the correct SSL portal URL to login."
  
		end
            end
        else
            local defaultPortalName = db.getAttribute ("SSLVPNPortalLayout","DefaultPortal","2","PortalLayoutName")
            if (defaultPortalName == nil) then
                defaultPortalName = db.getAttribute ("SSLVPNPortalLayout","DefaultPortal","1","PortalLayoutName")
            end
            if (defaultPortalName ~= portalName) then
                return 3, "User authentication Failed. Only default portal's users should be allowed to login."
            end
        end
--        local status, errMsg = sslvpnLuaLib.sslLoginPolicyVerify (username, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent)
--        if (status == -1) then
--            return 3, errMsg
--        end

	local httpReferer = login.parseReferer (os.getenv("HTTP_REFERER") or '')
        local loginState, uniqueVal, errorMsg, prev = sslvpnLuaLib.sslLogin (username, password, domainName, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent, SAPI.Request.servervariable("URL"), httpReferer)
        if (loginState == -1) then

		if (lockoutEnabled == "1") then
			-- If user fails to authenticate and failure count has reached max 
        	-- attempts allowed, user will be blocked for time period configured 
        	-- which will be calculated from last failed login attempt.
			local wherePart = "UserName = '" .. username .. "'"
       		local oldRow = db.existsRowWhere("UserLoginFailure", wherePart)
       		local fail_cnt = db.getAttribute("UserLoginFailure", "UserName", username, "FailureCount")
       		local tempTable = {}
       		tempTable["UserLoginFailure.UserName"] = username
       		if(fail_cnt ~= nil) then
	       		fail_cnt = fail_cnt + 1
                if (maxLockoutAttempts == tostring(fail_cnt)) then
                	-- update the lock time file and reset login failure count, 
                    -- everytime user reaches max login attempts allowed
                    os.execute("/pfrm2.0/bin/timeShow > /var/log/" .. username .. ".lockTime")
                    fail_cnt = 0 
                end
                tempTable["UserLoginFailure.FailureCount"] = fail_cnt
                if (oldRow) then
                	db.update("UserLoginFailure", tempTable, oldRow)
                end
	        elseif(fail_cnt == nil) then
            	tempTable["UserLoginFailure.FailureCount"] = 1
		        db.insert("UserLoginFailure", tempTable)
            end
		end

		if(userType == "3") then
            		return -1, "User Login Failed for Admin User."
		elseif(userType == "4") then
            		return -1, "User Login Failed for Guest User."
		elseif(userType == "0") then
			return -1, "User Login Failed for SSLVPN User."
		else
			return -1, "User Login Failed."
		end
        elseif (loginState == 0) then
            cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
			db.setAttribute("UserLoginFailure", "UserName", username, "FailureCount", 0)
            return 0,"OK",nil,uniqueVal
        elseif (loginState == -2) then
            return 1,errorMsg,tableInput
        elseif (loginState == -4) then
            cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
            return 2,errorMsg
        else
            return 3,errorMsg
        end
    end

	--- check for rows
    if(status == false) then
       statusCode = "LOGIN_INVALID_PASSWORD"
    end
	return -1, statusCode
end

function login.loginxml (tableInput)
	local username = tableInput["USERDBUsers.UserName"]
	local password = tableInput["USERDBUsers.Password"]
	local userAgent = tableInput["Login.userAgent"]
    local userConfig = db.getRow ("USERDBUsers","UserName",username)
    local status = false
    if (userConfig ~= nil) then
	local userType = db.getAttribute ("USERDBUsers", "UserName", username, "UserType")
        local groupName = db.getAttribute ("USERDBUsers", "UserName", username, "GroupName")
        local domainName = db.getAttribute ("USERDBGroups","GroupName",groupName,"DomainName")
        local domainName = db.getAttribute ("USERDBGroups","GroupName",groupName,"DomainName")
        local portalName = db.getAttribute ("USERDBDomains","DomainName",domainName,"PortalLayoutName")
        if (Page == SSL_PORTAL_LOGIN_PAGE) then
		local selectedPortalName = tableInput["Login.PortalName"]
            if (selectedPortalName == nil) then
                return 3, "Failed to find Portal Information"
            elseif (selectedPortalName ~= portalName) then
		if(userType == "3") then 
			return 3, "Admin User authentication Failed."
		elseif(userType == "4") then 
			return 3,"Guest User authentication Failed."
		elseif(userType == "0") then
                 	return 3, "SSLVPN User authentication Failed. Use the correct SSL portal URL to login."
		else
                 	return 3, "User authentication Failed. Use the correct SSL portal URL to login."
		end
            end
        else
            local defaultPortalName = db.getAttribute ("SSLVPNPortalLayout","DefaultPortal","2","PortalLayoutName")
            if (defaultPortalName == nil) then
                defaultPortalName = db.getAttribute ("SSLVPNPortalLayout","DefaultPortal","1","PortalLayoutName")
            end
            if (defaultPortalName ~= portalName) then
                return 3, "User authentication Failed. Only default portal's users should be allowed to login."
            end
        end
        local status, errMsg = sslvpnLuaLib.sslLoginPolicyVerify (username, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent)
        if (status == -1) then
            return 3, errMsg
        end
	local httpReferer = login.parseReferer (os.getenv("HTTP_REFERER"))
        local loginState, uniqueVal, errorMsg, prev = sslvpnLuaLib.sslLogin (username, password, domainName, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent, SAPI.Request.servervariable("URL"), httpReferer)
        if (loginState == -1) then
		if(userType == "3") then
            		return -1, "User Login Failed for Admin User."
		elseif(userType == "4") then
            		return -1, "User Login Failed for Guest User."
		elseif(userType == "0") then
			return -1, "User Login Failed for SSLVPN User."
		else
			return -1, "User Login Failed."
		end
        elseif (loginState == 0) then
            --cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
            return 0,"OK",nil,uniqueVal
        elseif (loginState == -2) then
            return 1,errorMsg,tableInput
        elseif (loginState == -4) then
            cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
            return 2,errorMsg
        else
            return 3,errorMsg
        end
    end

	--- check for rows
    if(status == false) then
       statusCode = "LOGIN_INVALID_PASSWORD"
    end
	return -1, statusCode
end

-- Function for forced login
function login.forcedLogin (tableInput)
	local username = tableInput["USERDBUsers.UserName"]
	local password = tableInput["USERDBUsers.Password"]
--	local userAgent = tableInput["Login.userAgent"]
	local userAgent = os.getenv("HTTP_USER_AGENT")
	local domainName = tableInput["USERDBDomains.Domainname"]
	local userConfig = db.getRow ("USERDBUsers","UserName",username)
	local status = false

      	if (userConfig ~= nil) then
        	local groupName = db.getAttribute ("USERDBUsers", "UserName", username, "GroupName")
        	local domainName = db.getAttribute ("USERDBGroups","GroupName",groupName,"DomainName")
        	local status, errMsg = sslvpnLuaLib.sslLoginPolicyVerify (username, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent)
        	if (status == -1) then
            	return 3, errMsg
        	end
        end

	    local httpReferer = login.parseReferer (os.getenv("HTTP_REFERER") or "")
        local loginState, uniqueVal, errorMsg, prev = sslvpnLuaLib.sslForcedLogin (username, password, domainName, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent, SAPI.Request.servervariable("URL"), httpReferer)
        if (loginState == -1) then
            return -1, "LOGIN_INVALID_PASSWORD"
        elseif (loginState == 0) then
            cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
            return 0, uniqueVal
        elseif (loginState == -4) then
            cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
            return 2, errorMsg
        else
            return 3, errorMsg
        end

	--- check for rows
    if(result == false) then
       statusCode = "LOGIN_INVALID_PASSWORD"
    end
	return -1, statusCode
end

-- Function for forced login
function login.forcedLoginXml (tableInput)
	local username = tableInput["USERDBUsers.UserName"]
	local password = tableInput["USERDBUsers.Password"]
	local userAgent = tableInput["Login.userAgent"]
    local userConfig = db.getRow ("USERDBUsers","UserName",username)
    local status = false
    if (userConfig ~= nil) then
        local groupName = db.getAttribute ("USERDBUsers", "UserName", username, "GroupName")
        local domainName = db.getAttribute ("USERDBGroups","GroupName",groupName,"DomainName")
        local status, errMsg = sslvpnLuaLib.sslLoginPolicyVerify (username, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent)
        if (status == -1) then
            return 3, errMsg
        end
        local loginState, uniqueVal, errorMsg, prev = sslvpnLuaLib.sslForcedLogin (username, password, domainName, SAPI.Request.servervariable("REMOTE_ADDR"), SAPI.Request.servervariable("SERVER_ADDR"), userAgent, SAPI.Request.servervariable("URL"))
        if (loginState == -1) then
            return -1, "LOGIN_INVALID_PASSWORD"
        elseif (loginState == 0) then
           -- cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
            return 0,"OK",nil,uniqueVal
        elseif (loginState == -4) then
            --cgilua.cookies.set("TeamF1Login", uniqueVal)            
            ACCESS_LEVEL = prev
            return 2, errorMsg
        else
            return 3, errorMsg
        end
    end

	--- check for rows
    if(result == false) then
       statusCode = "LOGIN_INVALID_PASSWORD"
    end
	return -1, statusCode
end

-- Revalidate current login before proceeding.
-- Returns 0 if login ok, 1 if no cookie
-- 2 if cookie but no session, 3 if cookie but
-- expired session, and 4 if only authenticated, not logged in.
function login.revalidate (cookieVal, ipaddr)
	-- if no cookie, go to login page
    if (cookieVal == nil) then
		util.appendDebugOut("PLEASE LOGIN - NOT LOGGED IN YET! - no cookie<br>")
		return "NO_COOKIE"
	else
		local wherePart = "LoginAddress = '" .. ipaddr .. "' AND Cookie = '" .. cookieVal .. "'"
		local row = db.getRowWhere("SSLVPNActiveUsers", wherePart)
        local status,upTime = timeLib.uptime()

		-- if no matching session
		if (row == nil) then
			util.appendDebugOut("PLEASE RELOGIN - LOGIN EXPIRED! - someone else killed your expired session<br>")
			statusMessage = db.getAttribute("stringsMap", "stringId", "LOGIN_SESSION_EXPIRED", LANGUAGE) or "NO_SESSION"
			return "NO_SESSION"
		-- logged in, so reset time
		else
            local status, errMsg, priv = sslvpnLuaLib.sslLoginSessionValidate(cookieVal, ipaddr)
            -- if expired login
            if (status == -1) then
    			util.appendDebugOut("PLEASE RELOGIN - LOGIN EXPIRED! - you waited too long<br>")
	    		statusMessage = db.getAttribute("stringsMap", "stringId", "LOGIN_SESSION_EXPIRED", LANGUAGE)
		    	return "EXPIRED_SESSION"                
            end
            local sessionStatus = row["SSLVPNActiveUsers.SessionStatus"]
            if (sessionStatus ~= "ACTIVE") then
	    		statusMessage = db.getAttribute("stringsMap", "stringId", sessionStatus, LANGUAGE)
                return "EXPIRED_SESSION"
            end
			util.appendDebugOut("YOU ARE ALREADY LOGGED IN!<br>")
			db.setAttributeWhere("SSLVPNActiveUsers", wherePart, "LastAccessTime", upTime)
			-- set access level
			ACCESS_LEVEL = priv
			return "OK"
		end
	end
end

-- Log out current user. Deletes session.
function login.logout (cookieVal, ipaddr)
	if (cookieVal ~= nil) then
		local wherePart = "LoginAddress = '" .. ipaddr .. "' AND Cookie = '" .. cookieVal .. "'"
		-- delete session
        local status =  sslvpnLuaLib.sslLogout(cookieVal)
        -- set prive levels
        ACCESS_LEVEL = 1 -- setting as GUEST
        -- delete activeUser's Table Entry
		db.deleteRowWhere("SSLVPNActiveUsers", wherePart)
        -- Clean the Cookie
        cgilua.cookies.set("TeamF1Login", cgilua.cookies.get("TeamF1Login"), {expires=-1})
		return 0 --success
	else
		return 1
	end
end

-- Log out current user. Deletes session.
function login.logoutxml (cookieVal, ipaddr)
	if (cookieVal ~= nil) then
		local wherePart = "LoginAddress = '" .. ipaddr .. "' AND Cookie = '" .. cookieVal .. "'"
		-- delete session
        local status =  sslvpnLuaLib.sslLogout(cookieVal)
        -- set prive levels
        ACCESS_LEVEL = 1 -- setting as GUEST
        -- delete activeUser's Table Entry
		db.deleteRowWhere("SSLVPNActiveUsers", wherePart)
        -- Clean the Cookie
        --cgilua.cookies.set("TeamF1Login", cgilua.cookies.get("TeamF1Login"), {expires=-1})
		return 0 --success
	else
		return 1
	end
end

-- validate referer value for restricted pages
function login.refererValidation (cookieVal, ipaddr, refererVal)
    if (cookieVal ~= nil and ipaddr ~= nil) then	
        local wherePart = "LoginAddress = '" .. ipaddr .. "' AND Cookie = '" .. cookieVal .. "'"
        local row = db.getRowWhere("SSLVPNActiveUsers", wherePart)

        if (row == nil) then
            return true
        elseif(row["SSLVPNActiveUsers.RefererValue"] ~= nil and row["SSLVPNActiveUsers.RefererValue"] ~= refererVal) then
            return false
        end
    end
    return true
end

-- parse the http referer value
function login.parseReferer (refererVal)
    local refValue = ""
    -- splitting referer val to get the ipaddress/domain name
    -- https://networkName:portNumb/ or http://networkName/
    local splitRefVal = util.split(refererVal,"//")
    if (splitRefVal ~= false) then
        if (splitRefVal[2] ~= nil) then
            local networkName = util.split(splitRefVal[2], "/")
            refValue = networkName[1] or ''
        end
    end

    return refValue
end


