-- timeCLI.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 28jan08,gnm modified error message text in input validation routines.
-- 28nov07 gnm written
--
-- Description
-- CLISH time LUA routines

-- load required library
require "teamf1lualib/time"

--
-- This routine save user password configuration information
function timeConfigInputVal (configRow)

    if configRow["ntp.useDefServers"] == "0" then
        if configRow["ntp.server1"] == nil then
	        printCLIError ("Invalid time-server name or IP address for server1")
            return false			
        end
    end
    if (configRow["ntp.resolvIpv6Address"] == "1" and db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") ~= "3") then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure this option.")
        return false
    end
    if (configRow["ntp.ntpMode"] == "0") then
        if (configRow["ntp.Stratum"] == "" or configRow["ntp.Stratum"] == nil) then
            printCLIError("Invalid Stratum.")
            return false
        end
        if (configRow["ntp.setManualDate"] == "1") then
            if (configRow["ManualDate.Year"] == nil or configRow["ManualDate.Year"] == nil) then
                printCLIError("Invalid ntp_year.")
                return false
            end
            if (configRow["ManualDate.Month"] == nil or configRow["ManualDate.Month"] == nil) then
                printCLIError("Invalid ntp_month.")
                return false
            end
            if (configRow["ManualDate.Day"] == nil or configRow["ManualDate.Day"] == nil) then
                printCLIError("Invalid ntp_day.")
                return false
            end
            if (configRow["ManualDate.hours"] == nil or configRow["ManualDate.hours"] == nil) then
                printCLIError("Invalid ntp_hour.")
                return false
            end
            if (configRow["ManualDate.minutes"] == nil or configRow["ManualDate.minutes"] == nil) then
                printCLIError("Invalid ntp_minutes.")
                return false
            end
            if (configRow["ManualDate.seconds"] == nil or configRow["ManualDate.seconds"] == nil) then
                printCLIError("Invalid ntp_seconds.")
                return false
            end
        end
    elseif (configRow["ntp.ntpMode"] == "2") then
        local vpnRow = {}
        if (configRow["ntp.VPNPolicy"] ~= nil) then
            vpnRow = db.getRow("IpsecVPNPolicy","VPNPolicyName",configRow["ntp.VPNPolicy"])
        end
        if (configRow["ntp.VPNPolicy"] == "" or configRow["ntp.VPNPolicy"] == nil or
           (configRow["ntp.VPNPolicy"] ~= "None" and vpnRow == nil)) then
            printCLIError("Invalid vpn_policy name.Please select from following VPN Policies")
            print("'None'")
            local vpnTable = db.getTable("IpsecVPNPolicy")
            local i = 0
            for k,v in pairs(vpnTable) do
                i = i + 1
                local vpn = vpnTable[i]
                if (vpn ~= nil) then
                    print("'"..vpn["IpsecVPNPolicy.VPNPolicyName"].."'")
                end
            end
            return false
        end
    end

    return true
end

--
-- This routine save user password configuration information
function timeConfigSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""

    if(configRow["ntp.resolvIpv6Address"] == "1") then
        configRow["ntp.networkMode"] = "6"
    else
        configRow["ntp.networkMode"] = "4"
    end
    configRow["ntp.resolvIpv6Address"] = nil
    -- Write configuration to database
    errorFlag, statusCode = time.ntp_config (configRow, 
	       configRow["ntp._ROWID_"], "edit")
	
    -- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function timeConfigInit (args)
    local configRow = {}
    local rowId = 1 -- NTP table only only one row

    configRow = db.getRow("ntp", "ntp._ROWID_", rowId)
    local timeRow = db.getRow("ManualDate","_ROWID_",rowId)
	if(timeRow ~= nil) then
		configRow["ManualDate.hours"] = timeRow["ManualDate.hours"]
		configRow["ManualDate.minutes"] = timeRow["ManualDate.minutes"]
		configRow["ManualDate.seconds"] = timeRow["ManualDate.seconds"]
		configRow["ManualDate.Day"] = timeRow["ManualDate.Day"]
		configRow["ManualDate.Month"] = timeRow["ManualDate.Month"]
		configRow["ManualDate.Year"] = timeRow["ManualDate.Year"]
	end
    -- clear fields we do not want with current values in database
    if(configRow["ntp.networkMode"] == "6" and db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") == "3") then
        configRow["ntp.resolvIpv6Address"] = "1"
    else
        configRow["ntp.resolvIpv6Address"] = "0"
    end
    return rowId, configRow
end
