require "teamf1lualib/util"
require "teamf1lualib/db"

tr69Glue = {}
db.connect("/tmp/system.db")
ACCESS_LEVEL = 0
SETTINGS_FILE=db.getAttribute("environment", "name", "TEAMF1_CFG_ASCII", "value")
if (SETTINGS_FILE == nil) then
	SETTINGS_FILE= "/flash/teamf1.cfg.ascii"
end

status = -1

local tr69GlueStaticMap = {}
local tr69GlueInstanceMap = {}
local sqlVars = {}

for line in io.lines ("/pfrm2.0/etc/tr69DB.map") do
    for sqlVar,value in string.gfind (line,"(.-)=(.*)") do
        sqlVars[sqlVar] = value
    end
    for param,table,field,sql,getFunc,setFunc in string.gfind (line,"(.*):(.*):(.*):(.*):(.*):(.*)") do
        tr69GlueStaticMap [param] = {}
        tr69GlueStaticMap [param].table = table
        tr69GlueStaticMap [param].field = field
        tr69GlueStaticMap [param].sql = sqlVars[sql] or sql
        tr69GlueStaticMap [param].getFunc = getFunc
        tr69GlueStaticMap [param].setFunc = setFunc
    end
end

function tr69Glue.AddToInstanceMap (name, rowId, staticParam)
    tr69GlueInstanceMap [name] = {}
    tr69GlueInstanceMap [name].rowId = rowId
    tr69GlueInstanceMap [name].staticParam = staticParam
end

function tr69Glue.rollbackTransaction ()
    db.rollback()
end

function tr69Glue.startTransaction ()
    db.beginTransaction()
end

function tr69Glue.commitTransaction ()
    db.commitTransaction()
end

-- generic set function
function tr69Glue.setField (intable, rowids)
    for table,value in pairs (intable) do
        status = db.update (table, value, rowids[table])
        print ("tr69Glue: updating " .. table)
        if (status == nil) then
            status = -1
            return;
        end
    end
	-- save the set operation.
	SETTINGS_FILE=db.getAttribute("environment", "name", "TEAMF1_CFG_ASCII", "value")
	db.save()
end
    
function tr69Glue.reboot (delay)
    db.setAttribute("reboot", "reboot._ROWID_", "1", "reboot", "1")
    db.setAttribute("reboot", "reboot._ROWID_", "1", "rebootTime", delay)
end

function tr69Glue.factoryReset()
   -- remove tr69 persistent data
   local cfg = db.getAttribute("environment", "name", "TR69_PERSISTENT_STORE", "value")
   if (cfg ~= nil) then
       os.execute ("rm -Rf " .. cfg)
    end       

   local fact_reset_program = db.getAttribute("environment", "name", "FACTORY_RESET_SCRIPT", "value")
   os.execute(fact_reset_program)

   -- send a signal to reboot
   db.setAttribute ("reboot", "reboot._ROWID_", "1", "reboot", "1")
end

function tr69Glue.downloadReset()

   local fact_reset_program = db.getAttribute("environment", "name", "TR69_DOWNLOAD_RESET_SCRIPT", "value")
   os.execute(fact_reset_program)

   -- send a signal to reboot
   db.setAttribute ("reboot", "reboot._ROWID_", "1", "reboot", "1")
end

function tr69Glue.FirmwareCheck (filename)
    require "teamf1lualib/firmware"

	-- verify if the firmware is valid
    status = firmware.upgrade(filename, 0, 0, 0, " -v -c ")
end

function tr69Glue.FirmwareUpgrade (filename)
    require "teamf1lualib/firmware"

	-- upgrade the firmware
	status = firmware.upgrade(filename, 1, 5, 1, " -n ")
end

function tr69Glue.configDownload (filename)
    require "teamf1lualib/config"


    status, errCode = config.restore (filename)
	-- remove tr69 persistent data
	local cfg = db.getAttribute("environment", "name", "TR69_PERSISTENT_STORE", "value")
	if (cfg ~= nil) then
		os.execute ("rm -Rf " .. cfg)
	end       
end

function tr69Glue.configUpload (filename)
    os.execute ("cp /tmp/teamf1.cfg.ascii " .. filename)
    status = 0
end

function tr69Glue.getLogs (filename)
    print ("in getlogs")
    local logs = db.getTable("eventLog", false)
    local s

    local file = io.open(filename,"w")

    for k,v in pairs(logs) do
        s = v["textMessage"]
        file:write(s)
    end
    file:close()
    status = 0
end
function tr69Glue.configBakup ()
    local filename = "/data/tmp/tr069.bk.cfg"
    local f = io.open(filename, "w")
    db.saveTable(f, "tr069Config", db.getTable("tr069Config", false), false, 1, 1)
    file:close(filename)
end
        
-- load external tr69 functions if available
status, errMsg = pcall(require, 'teamf1lualib/tr69funcs')
if (status == false) then
	print ("Error load tr69funcs: " .. errMsg)
end


