-- upnpCLI.lua
-- Mohammad Rafi
-- TeamF1
-- www.TeamF1.com
--
-- Modification History
-- 08Apr13,nik Fixed SPR 36652
-- 19Sep08,rfi written
--
-- Description
-- CLISH UPnP LUA routines

require "teamf1lualib/upnp"

--
-- get the field values from the dbase table 
function upnpPortmapGet()
    local prog = nil
    prog = db.getAttribute("environment", "name", "UPNP_REFRESH_PROGRAM", "value")
    if (prog ~= nil) then
    	os.execute(prog .. " " .. "/tmp/system.db")
    end

    local resultTab = {}   
    local table = db.getTable("upnpPortMap")

    for _, row in pairs(table) do
        resTab.insertField (resultTab, "Active", row["upnpPortMap.active"])
        resTab.insertField (resultTab, "Protocol", row["upnpPortMap.protocol"])
        resTab.insertField (resultTab, "Int. Port", row["upnpPortMap.intPort"])
        resTab.insertField (resultTab, "Ext. Port", row["upnpPortMap.extPort"])
        resTab.insertField (resultTab, "IP Address", row["upnpPortMap.ipAddr"])
    end

    printLabel ("UPnP Portmap Table")
    resTab.print (resultTab, 0)

end

--
-- get the field values from the dbase table 
function upnpGet()

    local resultTab = {}   
    local upnp = db.getAttribute ("upnp", "_ROWID_", 1, "upnpEnable")
    if (upnp ~= "1") then
        print ("\n" .. "UPnP is disabled" .. "\n")
    else
        local row = db.getRow ("upnp", "_ROWID_", 1)
        resTab.insertField (resultTab, "Advertisement Period", row["upnp.advPeriod"])
        resTab.insertField (resultTab, "Advertisement Time To Live", row["upnp.advTimeToLive"])
        printLabel ("UPnP configuration")
        resTab.print (resultTab, 0)
    end
end

--
-- This routine save UPnP configuration
function upnpConfigSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "upnp"
    -- input validation
    if (configRow["upnp.upnpEnable"] == "0") then
        upnpRow = db.getRow("upnp", "upnp._ROWID_", "1")
        if (upnpRow ~= nil) then
            if (configRow["upnp.advPeriod"] ~= upnpRow["upnp.advPeriod"]) then
                printCLIError("Please first enable upnp to configure advertisement period field")
                return false
            end
            if (configRow["upnp.advTimeToLive"] ~= upnpRow["upnp.advTimeToLive"]) then
                printCLIError("Please first enable upnp to configure advertisement Time to live field")
                return false
            end
        end
    end

    -- Write configuration to database
    errorFlag, statusCode = upnp.config (configRow,configRow["upnp._ROWID_"], "edit")
    if (errorFlag == "OK") then
        db.save()
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function upnpConfigInit (args)
    local configRow = {}
    local rowId = 1 -- UPNP table only only one row

    configRow = db.getRow("upnp", "upnp._ROWID_", rowId)
    return rowId, configRow
end
